/*
   Copyright (c) 1995-8,2001  Martin Schulze <joey@infodrom.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.
*/

/*
**  Write log into a file and not via syslog
*/

#ifndef __log_h__
#define __log_h__

void
openlogfile(char *name);

void
closelogfile();

/* Writes into the logfile, using variable number of arguments.
 */
void
filelog(char *format, ...);

#endif /* __log_h__ */
